import sys
import pytest

from ewoks.__main__ import main
from ewokscore import load_graph
from ewokscore.tests.examples.graphs import graph_names
from ewokscore.tests.examples.graphs import get_graph
from ewoksppf.tests.test_examples import assert_results as assert_ppf_results
from ewoksdask.tests.test_examples import assert_all_results as assert_dask_results
from ewokscore.tests.test_examples import assert_all_results as assert_core_results


@pytest.mark.parametrize("graph_name", graph_names())
@pytest.mark.parametrize("scheme", (None, "json"))
@pytest.mark.parametrize("binding", ("none", "dask", "ppf"))
def test_main_execute(graph_name, scheme, binding, tmpdir):
    if scheme == "json":
        pytest.skip("TODO")
    graph, expected = get_graph(graph_name)
    argv = [
        sys.executable,
        "execute",
        graph_name,
        "--test",
        "--binding",
        binding,
        "--output",
        "all",
    ]
    if scheme:
        argv += ["--data-root-uri", str(tmpdir), "scheme", scheme]
        varinfo = {"root_uri": str(tmpdir), "scheme": scheme}
    else:
        varinfo = None

    ewoksgraph = load_graph(graph)
    non_dag = ewoksgraph.is_cyclic or ewoksgraph.has_conditional_links

    if non_dag and binding != "ppf":
        with pytest.raises(RuntimeError):
            main(argv=argv, shell=False)
        return

    result = main(argv=argv, shell=False)

    if binding == "ppf":
        assert_ppf_results(graph, ewoksgraph, result, expected, varinfo)
    elif binding == "dask":
        assert_dask_results(ewoksgraph, result, expected, varinfo)
    else:
        assert_core_results(ewoksgraph, result, expected, varinfo)



import sys
import argparse
import logging

from pprint import pformat
from ewokscore import cliutils
from .bindings import execute_graph

logger = logging.getLogger(__name__)


def create_argument_parser(shell=False):
    parser = argparse.ArgumentParser(
        description="Esrf WOrKflow Sytem CLI", prog="ewoks"
    )

    subparsers = parser.add_subparsers(help="Commands", dest="command")
    execute = subparsers.add_parser("execute", help="Execute a workflow")

    execute.add_argument(
        "--binding",
        type=str,
        choices=["none", "dask", "ppf", "orange"],
        default="none",
        help="Task binding to be used",
    )
    cliutils.add_execute_parameters(execute, shell=shell)

    return parser


def command_execute(args, shell=False):
    cliutils.apply_execute_parameters(args, shell=shell)
    results = execute_graph(args.graph, binding=args.binding, **args.execute_options)
    logger.info("Results for workflow '%s': \n%s", args.workflow, pformat(results))
    if shell:
        if results is None:
            return 1
        else:
            return 0
    else:
        return results


def command_default(args, shell=False):
    if shell:
        return 0
    else:
        return None


def main(argv=None, shell=True):
    parser = create_argument_parser(shell=shell)

    if argv is None:
        argv = sys.argv
    args, _ = parser.parse_known_args(argv[1:])

    if args.command == "execute":
        return command_execute(args, shell=shell)
    else:
        return command_default(args, shell=shell)


if __name__ == "__main__":
    sys.exit(main())



import os
import importlib
import tempfile
from typing import Optional, List
from contextlib import contextmanager


def import_binding(binding: Optional[str]):
    if not binding or binding.lower() == "none":
        binding = "ewokscore"
    elif not binding.startswith("ewoks"):
        binding = "ewoks" + binding
    return importlib.import_module(binding)


@contextmanager
def convert_to_ewoks_graph(graph, binding: Optional[str], **load_graph_options):
    if isinstance(graph, str) and graph.endswith(".ows") and binding != "orange":
        mod = importlib.import_module("ewoksorange.bindings")
        with tempfile.TemporaryDirectory(prefix="ewoks") as tmpdirname:
            filename = os.path.join(tmpdirname, "ewokstaskgraph.json")
            yield mod.ows_to_ewoks(graph, filename, **load_graph_options)
    else:
        yield graph


def execute_graph(
    graph,
    binding: Optional[str] = None,
    inputs: Optional[List[dict]] = None,
    load_options: Optional[dict] = None,
    **execute_options
):
    with convert_to_ewoks_graph(
        graph, binding, inputs=inputs, load_options=load_options
    ) as graph:
        mod = import_binding(binding)
        return mod.execute_graph(
            graph, inputs=inputs, load_options=load_options, **execute_options
        )
